/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/


#include <cassert>

#include <core_api/AppContext.h>
#include <datatype/DNASequence.h>
#include <core_api/DNAAlphabet.h>
#include <datatype/MAlignment.h>

#include "RemoteMachine.h"
#include "SerializeUtils.h"

namespace GB2 {

QVariant SerializeUtils::serializeString( const char * data ) {
    QVariantList res;
    if( NULL == data ) {
        res << -1 << 0;
    } else {
        QByteArray str( data );
        res << str.size() << str;
    }
    return res;
}

bool SerializeUtils::deserializeString( const QVariant & data, QByteArray * arr ) {
    if( NULL == arr ) {
        return false;
    }
    if( !data.canConvert( QVariant::List ) ) {
        return false;
    }
    QVariantList args = data.toList();
    if( 2 != args.size() ) {
        return false;
    }
    if( -1 == args[0].toInt() ) {
        *arr = QByteArray();
        return true;
    }
    
    if( !args[1].canConvert( QVariant::ByteArray ) ) {
        return false;
    }
    *arr = args[1].toByteArray();
    assert( args[0].toInt() == arr->size() );
    return true;
}

QVariant SerializeUtils::serializeDNASequence( const DNASequence & sequence ) {
    assert( !sequence.isNull() && NULL != sequence.alphabet );
    QVariantList res;
    res << sequence.info << sequence.seq << serializeDNAAlphabet( sequence.alphabet );
    return res;
}

bool SerializeUtils::deserializeDNASequence( const QVariant & data, DNASequence * seq ) {
    if( NULL == seq ) {
        return false;
    }
    if( !data.canConvert( QVariant::List ) ) {
        return false;
    }
    QVariantList args = data.toList();
    if( SERIALIZED_DNASEQUENCE_LIST_SZ != args.size() ) {
        return false;
    }

    if( !args[0].canConvert( QVariant::Map ) ) {
        return false;
    }
    seq->info = args[0].toMap();

    if( !args[1].canConvert( QVariant::ByteArray ) ) {
        return false;
    }
    seq->seq = args[1].toByteArray();
    
    if( !deserializeDNAAlphabet( args[2], &seq->alphabet ) ) {
        return false;
    }
    assert( NULL != seq->alphabet );
    
    return true;
}

QVariant SerializeUtils::serializeDNAAlphabet( DNAAlphabet * al ) {
    QVariantList res;
    if( NULL != al ) {
        res << true << al->getId();
    } else {
        res << false << "";
    }
    return res;
}

bool SerializeUtils::deserializeDNAAlphabet( const QVariant & data, DNAAlphabet ** al ) {
    if( NULL == al ) {
        return false;
    }
    *al = NULL;
    if( !data.canConvert( QVariant::List ) ) {
        return false;
    }
    
    QVariantList args = data.toList();
    if( 2 != args.size() || !args[0].canConvert( QVariant::Bool ) || !args[1].canConvert( QVariant::String ) ) {
        return false;
    }
    
    if( true == args[0].toBool() ) {
        DNAAlphabetRegistry * alReg = AppContext::getDNAAlphabetRegistry();
        assert( NULL != alReg );
        *al = alReg->findById( args[1].toString() );
    }
    return true;
}

static QVariant serializeMalignmentItems( const QList< MAlignmentItem > & items ) {
    QVariantList res;
    foreach( MAlignmentItem item, items ) {
        QVariantList resItem;
        resItem << item.name << item.sequence;
        res << QVariant( resItem );
    }
    return res;
}

static bool deserializeMalignmentItems( const QVariant & data, QList< MAlignmentItem > & items ) {
    if( !data.canConvert( QVariant::List ) ) {
        return false;
    }
    QVariantList args = data.toList();
    
    foreach( const QVariant & arg, args ) {
        if( !arg.canConvert( QVariant::List ) ) {
            items.clear();
            return false;
        }
        QVariantList itemArgs = arg.toList();
        if( 2 != itemArgs.size() || !itemArgs[0].canConvert( QVariant::String ) || !itemArgs[1].canConvert( QVariant::ByteArray ) ) {
            items.clear();
            return false;
        }
        MAlignmentItem item;
        item.name = itemArgs[0].toString();
        item.sequence = itemArgs[1].toByteArray();
        items << item;
    }
    return true;
}

QVariant SerializeUtils::serializeMAlignment( const MAlignment & ma ) {
    QVariantList res;
    res << serializeDNAAlphabet( ma.alphabet ) << QVariant( ma.info ) << serializeMalignmentItems( ma.alignedSeqs );
    return res;
}

bool SerializeUtils::deserializeMAlignment( const QVariant & data, MAlignment * ma ) {
    if( NULL == ma ) {
        return false;
    }
    if( !data.canConvert( QVariant::List ) ) {
        return false;
    }
    QVariantList args = data.toList();
    if( SERIALIZED_MALIGNMENT_LIST_SZ != args.size() ) {
        return false;
    }
    
    if( !deserializeDNAAlphabet( args[0], &ma->alphabet ) ) {
        return false;
    }
    
    if( !args[1].canConvert( QVariant::Map ) ) {
        return false;
    }
    ma->info = args[1].toMap();
    
    if( !deserializeMalignmentItems( args[2], ma->alignedSeqs ) ) {
        return false;
    }
    return true;
}

const QString SerializeUtils::REMOTE_MACHINE_VERSION_1_0 = "Remote machine v1.0: ";

QString SerializeUtils::serializeRemoteMachineSettings( RemoteMachineSettings * machine ) {
    assert( NULL != machine );
    return REMOTE_MACHINE_VERSION_1_0 + machine->getProtocolId() + " " + machine->serialize();
}

bool SerializeUtils::deserializeRemoteMachineSettingsFromFile( const QString & machinePath, RemoteMachine ** machine ) {
    QFile file( machinePath );
    if( !file.open( QIODevice::ReadOnly ) ) {
        return NULL;
    }
    
    QDataStream in( &file );
    QString data;
    in >> data;
    
    if( !deserializeRemoteMachineSettings( data, machine ) || NULL == machine ) {
        return false;
    }
    return true;
}

static QString getProtocolId( const QString & data ) {
    QString res;
    QList< ProtocolInfo* > infos = AppContext::getProtocolInfoRegistry()->getProtocolInfos();
    foreach( ProtocolInfo * pi, infos ) {
        assert( NULL != pi );
        if( data.startsWith( pi->getId() ) ) {
            res = pi->getId();
            break;
        }
    }
    return res;
}

bool SerializeUtils::deserializeRemoteMachineSettings( const QString & data, RemoteMachineSettings ** ret, QString * retProtoId ) {
    if( NULL == ret ) {
        return false;
    }
    *ret = NULL;
    
    if( data.startsWith( REMOTE_MACHINE_VERSION_1_0 ) ) {
        QString machineData = data.mid( REMOTE_MACHINE_VERSION_1_0.size() );
        QString protoId = getProtocolId( machineData );
        ProtocolInfo * protoInfo = AppContext::getProtocolInfoRegistry()->getProtocolInfo( protoId );
        if( NULL == protoInfo ) {
            return false;
        }
        
        *ret = protoInfo->getRemoteMachineFactory()->createSettings( machineData.mid( protoId.size() + 1 ) );
        if( NULL != retProtoId ) {
            *retProtoId = protoId;
        }
        return NULL == *ret ? false : true;
    } else { /* there is only one version of remote machines in this version of ugene */
        return false;
    }
    
    return false; // unreachable code
}

bool SerializeUtils::deserializeRemoteMachineSettings( const QString & data, RemoteMachine ** machine ) {
    if( NULL == machine ) {
        return false;
    }
    *machine = NULL;
    
    RemoteMachineSettings * settings = NULL;
    QString protoId;
    if( !deserializeRemoteMachineSettings( data, &settings, &protoId ) ) {
        assert( NULL == settings );
        return false;
    }
    assert( NULL != settings );
    *machine = AppContext::getProtocolInfoRegistry()->getProtocolInfo( protoId )->getRemoteMachineFactory()->createInstance( settings );
    delete settings;
    
    return NULL == *machine ? false : true;
}

} // GB2
