/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "CreateSubalignimentDialogController.h"

#include <util_gui/DialogUtils.h>
#include <document_format/GenbankLocationParser.h>
#include <core_services/DocumentFormatRegistryImpl.h>
#include <core_api/AppContext.h>
#include <core_api/IOAdapter.h>

#include <QtGui/qfiledialog.h>
#include <QtGui/qmessagebox.h>

namespace GB2{

CreateSubalignimentDialogController::CreateSubalignimentDialogController(MAlignmentObject *_mobj, QWidget *p)
: QDialog(p), mobj(_mobj){
	setupUi(this);

	connect(browseButton, SIGNAL(clicked()), SLOT(sl_browseButtonClicked()));
	connect(allButton, SIGNAL(clicked()), SLOT(sl_allButtonClicked()));
	connect(noneButton, SIGNAL(clicked()), SLOT(sl_noneButtonClicked()));
	connect(invertButton, SIGNAL(clicked()), SLOT(sl_invertButtonClicked()));

	int rowNumbers = mobj->getMAlignment().alignedSeqs.size();

	sequencesTableWidget->clearContents();
	sequencesTableWidget->setRowCount(rowNumbers);
	sequencesTableWidget->setColumnCount(1);
	sequencesTableWidget->verticalHeader()->setHidden( true );
	sequencesTableWidget->horizontalHeader()->setHidden( true );
	sequencesTableWidget->setShowGrid(false);
	sequencesTableWidget->horizontalHeader()->setResizeMode( 0, QHeaderView::Stretch );
	
	for (int i=0; i<rowNumbers; i++) {
		QCheckBox *cb = new QCheckBox(mobj->getMAlignment().alignedSeqs.at(i).name, this);
		cb->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
		cb->setChecked(true);
		sequencesTableWidget->setCellWidget(i, 0, cb);
		sequencesTableWidget->setRowHeight(i, 15);
	}

	DocumentFormatRegistryImpl *dfri = new DocumentFormatRegistryImpl();
	QList<DocumentFormatId> dfIdList = dfri->getRegisteredFormats(), supportedFormats;
	foreach(DocumentFormatId dfId, dfIdList){
		DocumentFormat *df = AppContext::getDocumentFormatRegistry()->getFormatById(dfId);
		IOAdapterFactory* iof = AppContext::getIOAdapterRegistry()->getIOAdapterFactoryById(BaseIOAdapters::MEMORY_BUF);
		Document *d = df->createNewDocument(iof, "fake");
		bool supported = df->isObjectOpSupported(d, DocumentFormat::DocObjectOp_Add, GObjectTypes::MULTIPLE_ALIGNMENT);
		if(supported){
			foreach(QString ext, df->getSupportedDocumentFileExtensions()){
				filter.append("*." + ext + " ");
			}
		}
	}
}

void CreateSubalignimentDialogController::sl_browseButtonClicked(){
	LastOpenDirHelper h;

	QString newPath = QFileDialog::getSaveFileName(this, tr("Select file to save..."), h.dir, filter );
	filepathEdit->setText(newPath);
}

void CreateSubalignimentDialogController::sl_allButtonClicked(){
	for (int i=0; i<sequencesTableWidget->rowCount(); i++) {
		QCheckBox *cb = qobject_cast<QCheckBox*>(sequencesTableWidget->cellWidget(i, 0));
		cb->setChecked(true);
	}
}

void CreateSubalignimentDialogController::sl_invertButtonClicked(){
	for (int i=0; i<sequencesTableWidget->rowCount(); i++) {
		QCheckBox *cb = qobject_cast<QCheckBox*>(sequencesTableWidget->cellWidget(i, 0));
		cb->setChecked(!cb->isChecked());
	}
}

void CreateSubalignimentDialogController::sl_noneButtonClicked(){
	for (int i=0; i<sequencesTableWidget->rowCount(); i++) {
		QCheckBox *cb = qobject_cast<QCheckBox*>(sequencesTableWidget->cellWidget(i, 0));
		cb->setChecked(false);
	}
}

void CreateSubalignimentDialogController::accept(){
	QFileInfo fi(filepathEdit->text());
	QDir dirToSave(fi.dir());
	if (!dirToSave.exists()){
		QMessageBox::critical(this, this->windowTitle(), tr("Directory to save is not exists"));
		return;
	}
	if(filepathEdit->text().isEmpty()){
		QMessageBox::critical(this, this->windowTitle(), tr("Entered save filepath is empty"));
		return;
	}
	if(fi.baseName().isEmpty()){
		QMessageBox::critical(this, this->windowTitle(), tr("Filename to save is empty"));
		return;
	}

	int seqLen = mobj->getMAlignment().alignedSeqs.first().sequence.length();

	QString startStr = startPosEdit->text(), endStr = endPosEdit->text();
	bool ok, ok2;
	int start = startStr.toInt(&ok), end = endStr.toInt(&ok2);
	if(!ok){
		QMessageBox::critical(this, this->windowTitle(), tr("Start position must be integer"));
		return;
	}
	if(!ok2){
		QMessageBox::critical(this, this->windowTitle(), tr("End position must be integer"));
		return;
	}
	LRegion region(start, end - start), sequence(0, seqLen);
	if(!sequence.contains(region)){
		QMessageBox::critical(this, this->windowTitle(), tr("Entered region not contained in current sequence"));
		return;
	}

	selectSeqNames();

	if(selectedNames.size() == 0){
		QMessageBox::critical(this, this->windowTitle(), tr("You must select at least one sequence"));
		return;
	}

	window = region;

	this->close();
	QDialog::accept();
}

void CreateSubalignimentDialogController::selectSeqNames(){
	QStringList names;
	for (int i=0; i<sequencesTableWidget->rowCount(); i++) {
		QCheckBox *cb = qobject_cast<QCheckBox*>(sequencesTableWidget->cellWidget(i, 0));
		if(cb->isChecked()){
			names.append(cb->text());
		}
	}
	selectedNames = names;
}

};
