/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "GraphicsBranchItem.h"
#include "GraphicsButtonItem.h"
#include <QtGui/QPainter>
#include <QtGui/QPen>
#include <QtCore/QStack>
#include <QtGui/QGraphicsScene>
#include <gobjects/PhyTreeObject.h>
#include <core_api/AppContext.h>

namespace GB2 {

const qreal GraphicsBranchItem::DEFAULT_WIDTH = 25.0;
const int GraphicsBranchItem::TEXT_SPACE = 8;

BranchCreationTask::BranchCreationTask(PhyNode *n, const QFont &f, const QMap<PhyNode*, int> &a, int h):
    Task("Generating tree view", TaskFlag_None), node(n), font(f), aligns(a), size(0),
    minx(GraphicsBranchItem::DEFAULT_WIDTH), height(h) { tpm = Progress_Manual; }

void BranchCreationTask::initPos(GraphicsBranchItem *item) {
    int oy = item->scenePos().y();
    if (vers.contains(oy)) {
        GraphicsBranchItem *t = vers[oy];
        if (t) {
            QGraphicsItem *p = t->parentItem();
            t->setYW(p ? p->mapFromScene(0, oy).y() : oy);
        }
    }
    vers[oy] = NULL;
}

GraphicsBranchItem* BranchCreationTask::getNode(PhyNode *node) {
    if (isCanceled() || stateInfo.hasErrors())
        return NULL;

    int branches = node->branches.size();
    if (branches == 1 && (node->name=="" || node->name=="ROOT")) {
        assert(node != node->branches[0]->node2);
        return getNode(node->branches[0]->node2);
    }
    if (branches > 1) {
        stateInfo.progress = 100 * ++size / aligns.size();
        QList<GraphicsBranchItem*> items;
        int ind = -1;
        for (int i = 0; i < branches; ++i) {
            if (node->branches[i]->node2 != node) {
                GraphicsBranchItem *item = getNode(node->branches[i]->node2);
                if (item)
                    items.append(item);
                else
                    return NULL;
            } else
                ind = i;
        }

        GraphicsBranchItem *item = ind == -1 ? new GraphicsBranchItem()
            : new GraphicsBranchItem(font, node->branches[ind]->distance);
        int size = items.size();
        assert(size > 0);

        {
            int xmin = 0, ymin = items[0]->pos().y(), ymax = 0;
            for (int i = 0; i < size; ++i) {
                QPointF pos1 = items[i]->pos();
                if (pos1.x() < xmin)
                    xmin = pos1.x();
                if (pos1.y() < ymin)
                    ymin = pos1.y();
                if (pos1.y() > ymax)
                    ymax = pos1.y();
            }
            do {
                xmin -= GraphicsBranchItem::DEFAULT_WIDTH;
            } while (xmin > minx);

            minx = xmin;
            int d = ymax - ymin;
            int c = d / 2;
            int old_y = (ymax + ymin) / 2;
            int y = old_y;
            while (vers.contains(y)) {
                c += d;
                if (c >= ymax - ymin) {
                    if (d <= 1) {
                        y = old_y;
                        break;
                    }
                    d /= 2;
                    c = d / 2;
                }
                if ((item->direction == GraphicsBranchItem::up && ymin + c < old_y) ||
                        (item->direction == GraphicsBranchItem::down && ymin + c > old_y))
                    continue;
                y = ymin + c;
            }
            vers.insert(y, item);
            item->setPos(xmin, y);

            for (int i = 0; i < size; ++i) {
                items[i]->setDirection(items[i]->pos().y() > y ? GraphicsBranchItem::up : GraphicsBranchItem::down);
                items[i]->setParentItem(item);
                QRectF rect = items[i]->distanceText->boundingRect();
                items[i]->distanceText->setPos(-(items[i]->width + rect.width()) / 2, 0);
            }
        }
        return item;
    } else {
        QMap<PhyNode*, int>::iterator it = aligns.find(node);
        if (it != aligns.end()) {
            int y = (it.value() + 0.5) * height;
            GraphicsBranchItem *item = branches != 1 ? new GraphicsBranchItem(font, 0, y, node->name)
                : new GraphicsBranchItem(font, 0, y, node->name, node->branches[0]->distance);
            initPos(item);
            return item;
        }
        stateInfo.setError(tr("Tree and alignment are not equal"));
        return NULL;
    }
}

void BranchCreationTask::run() {
    for (QMap<PhyNode*, int>::iterator it = aligns.begin(), end = aligns.end(); it != end; ++it)
        vers.insert((it.value() + 0.5) * height, NULL);
    GraphicsBranchItem *item = getNode(node);
    stateInfo.progress = 100;
    if (item)
        emit si_calculated(item);
}

void GraphicsBranchItem::collapse() {
    collapsed = !collapsed;
    QList<QGraphicsItem*> items = childItems();
    if (collapsed) {
        int xmin = 0;
        qreal ymin = items[0]->pos().y(), ymax = 0;
        for (int i = 0, s = items.size(); i < s; ++i) {
            if (!dynamic_cast<GraphicsBranchItem*>(items[i]))
                continue;
            QPointF pos1 = items[i]->pos();
            if (pos1.x() < xmin || xmin == 0)
                xmin = pos1.x();
            if (pos1.y() < ymin)
                ymin = pos1.y();
            if (pos1.y() > ymax)
                ymax = pos1.y();
            if (items[i] != distanceText && items[i] != nameText) {
                items[i]->hide();
            }
        }
        if (xmin >= 2 * GraphicsBranchItem::DEFAULT_WIDTH)
            xmin /= 2;
        if (xmin < GraphicsBranchItem::DEFAULT_WIDTH)
            xmin = GraphicsBranchItem::DEFAULT_WIDTH;

        QPen pen1(QColor(0, 0, 0));
        pen1.setWidth(2);
        const int defHeight = 50;
        QGraphicsRectItem *r = new QGraphicsRectItem(0, -defHeight / 2, xmin, defHeight, this);
        r->setPen(pen1);
    } else {
        for (int i = 0, s = items.size(); i < s; ++i) {
            if (dynamic_cast<QGraphicsRectItem*>(items[i])) {
                items[i]->setParentItem(NULL);
                scene()->removeItem(items[i]);
            } else {
                if (items[i] != distanceText && items[i] != nameText) {
                    items[i]->show();
                }
            }
        }
    }
}

void GraphicsBranchItem::setSelected(bool sel) {
    QStack<GraphicsBranchItem*> stack;
    stack.push(this);
    while (!stack.empty()) {
        GraphicsBranchItem *top = stack.pop();
        if (top->button)
            top->button->setSelected(sel);
        QList<QGraphicsItem*> items = top->childItems();
        for (int i = 0, s = items.size(); i < s; ++i) {
            GraphicsBranchItem *item = dynamic_cast<GraphicsBranchItem*>(items[i]);
            if (item)
                stack.push(item);
        }
    }
}

GraphicsBranchItem::GraphicsBranchItem(const QFont &font, qreal x, qreal y, QString name)
: direction(GraphicsBranchItem::up), visible(true), collapsed(false), button(NULL), distanceText(NULL) {
    initText(font, name);
    setPen(QColor(0, 0, 0));
    setPos(x, y);
}

GraphicsBranchItem::GraphicsBranchItem(const QFont &font, qreal x, qreal y, QString name, qreal d)
: direction(GraphicsBranchItem::up), visible(true), collapsed(false), button(NULL) {
    initText(font, name);
    initText(font, d);
    QPen pen1(QColor(0, 0, 0));
    if (d < 0)
        pen1.setStyle(Qt::DashLine);
    setPen(pen1);
    setPos(x, y);
}

GraphicsBranchItem::GraphicsBranchItem(): direction(GraphicsBranchItem::up),
visible(true), collapsed(false), button(new GraphicsButtonItem()), distanceText(NULL), nameText(NULL) {
    setBrush(QColor(0, 0, 0));
    setPen(QColor(0, 0, 0));
    button->setParentItem(this);
}

GraphicsBranchItem::GraphicsBranchItem(const QFont &font, qreal d)
: direction(GraphicsBranchItem::up), visible(true), collapsed(false), button(new GraphicsButtonItem()), nameText(NULL) {
    initText(font, d);
    QPen pen1(QColor(0, 0, 0));
    if (d < 0)
        pen1.setStyle(Qt::DashLine);
    setPen(pen1);
    setBrush(QColor(0, 0, 0));
    setPen(QColor(0, 0, 0));
    button->setParentItem(this);
}

void GraphicsBranchItem::initText(const QFont &font1, QString name) {
    QFont font = font1;
    font.setPointSize(font.pointSize() - 2);
    nameText = new QGraphicsSimpleTextItem(name);
    nameText->setFont(font);
    nameText->setBrush(Qt::darkGray);
    QRectF rect = nameText->boundingRect();
    nameText->setPos(GraphicsBranchItem::TEXT_SPACE, -rect.height() / 2);
    nameText->setParentItem(this);
    nameText->setZValue(1);
}

void GraphicsBranchItem::initText(const QFont &font1, qreal d) {
    QFont font = font1;
    font.setPointSize(font.pointSize() - 2);
    QString str = QString::number(d, 'f', 3);
    int i = str.length() - 1;
    for (; i >= 0 && str[i] == '0'; --i) ;
    if (str[i] == '.')
        --i;
    str.truncate(i + 1);

    distanceText = new QGraphicsSimpleTextItem(str);
    distanceText->setFont(font);
    distanceText->setBrush(Qt::darkGray);
    QRectF rect = distanceText->boundingRect();
    distanceText->setPos(-rect.width(), 0);
    distanceText->setParentItem(this);
    distanceText->setZValue(1);
}

void GraphicsBranchItem::setYW(int new_y) {
    int old_y = pos().y();
    setPos(pos().x(), new_y);
    setHeightW(getHeight() + (direction == down ? old_y - new_y : new_y - old_y));

    QList<QGraphicsItem*> items = childItems();
    for (int i = 0, s = items.size(); i < s; ++i) {
        GraphicsBranchItem *item = dynamic_cast<GraphicsBranchItem*>(items[i]);
        if (item)
            item->setYW(item->pos().y() + old_y - new_y);
    }
}

void GraphicsBranchItem::setParentItem(QGraphicsItem *item) {
    prepareGeometryChange();
    width = pos().x() - item->pos().x();
    height = direction == up ? pos().y() - item->pos().y() : item->pos().y() - pos().y();
    setPos(width, direction == up ? height : -height);
    QAbstractGraphicsShapeItem::setParentItem(item);
}

void GraphicsBranchItem::setDirection(Direction d) {
    prepareGeometryChange();
    direction = d;
}

QPainterPath GraphicsBranchItem::shape() const {
    return QPainterPath();
}

QRectF GraphicsBranchItem::boundingRect() const {
    return QRectF(- width - 0.5, direction == up ? -height : -0.5, width + 0.5, height + 0.5);
}

void GraphicsBranchItem::paint(QPainter *painter, const QStyleOptionGraphicsItem*, QWidget*) {
    if (!visible)
        return;
    painter->setPen(pen());
    painter->drawLine(QPointF(0, 0), QPointF(-width, 0));
    painter->drawLine(QPointF(-width, 0), QPointF(-width, direction == up ? -height : height));
}

void GraphicsBranchItem::setWidth(qreal w) {
    if (width == w)
        return;

    setPos(pos().x() - width + w, pos().y());

    prepareGeometryChange();
    width = w;
}

void GraphicsBranchItem::setHeight(qreal h) {
    if (height == h)
        return;

    if (direction == up)
        setPos(pos().x(), pos().y() - height + h);

    prepareGeometryChange();
    height = h;
}

}//namespace
