/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "MSAEditorTreeViewer.h"
#include "MSAEditor.h"
#include "MSAEditorSequenceArea.h"
#include "GraphicsBranchItem.h"
#include <QtGui/QVBoxLayout>
#include <QtGui/QMouseEvent>
#include <QtCore/QStack>
#include <QtGui/QPrinter>
#include <QtGui/QPrintDialog>
#include <QtGui/QPainter>
#include <QtGui/QFileDialog>
#include <QMessageBox>

#include <core_api/ProjectModel.h>
#include <core_api/IOAdapter.h>
#include <core_api/DocumentModel.h>
#include <core_api/DocumentFormats.h>
#include <phyltree/PhyTreeGeneratorRegistry.h>
#include <core_api/AppContext.h>
#include <util_gui/DialogUtils.h>
#include <phyltree/CreatePhyTreeDialogController.h>
#include <iostream>

namespace GB2 {

#define IMAGE_DIR "image"

int MSAEditorTreeViewer::initialWidth = 420;

MSAEditorTreeViewer::MSAEditorTreeViewer(MSAEditorUI* _ui, QScrollBar *vb)
: vBar(vb), editor(_ui->editor), ui(_ui), root(NULL), treeGeneratorTask(NULL), maxNameWidth(0) {
    showNameLabels = editor->getMSAObject() == NULL;

    connect(editor, SIGNAL(si_buildStaticToolbar(GObjectView*, QToolBar*)), SLOT(sl_buildStaticToolbar(GObjectView*, QToolBar*)));
    connect(editor, SIGNAL(si_fontChanged(const QFont&)), SLOT(sl_recomputeView()));
    if (editor->getMSAObject()) {
        connect(ui->seqArea, SIGNAL(si_startChanged(const QPoint& , const QPoint& )), SLOT(sl_startChanged(const QPoint& , const QPoint&)));
        connect(editor->getMSAObject(), SIGNAL(si_alignmentChanged(const MAlignment&, const MAlignmentModInfo&)),
            SLOT(sl_alignmentChanged(const MAlignment&, const MAlignmentModInfo&)) );
        connect(ui->getUndoAction(), SIGNAL(triggered()), SLOT(sl_recomputeView()));
        connect(ui->getRedoAction(), SIGNAL(triggered()), SLOT(sl_recomputeView()));
    }
    setFrameShape(QFrame::NoFrame);

    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setAlignment(Qt::AlignRight | Qt::AlignTop);
    setScene(new QGraphicsScene());

    if (editor->getPhyObject()) {
        recomputeView(true);
    }
}

void MSAEditorTreeViewer::sl_showTree() {
    if (treeGeneratorTask->getState() == Task::State_Finished) {
        if (treeGeneratorTask->hasErrors() == false) {
            if (editor->getPhyObject() == NULL) {
                QList<int> sizes = ui->splitter->sizes();
                int new_w = sizes[2] + sizes[0] - initialWidth;
                if (sizes[2] < 0) {
                    sizes[0] += sizes[2];
                    sizes[2] = 0;
                } else {
                    sizes[0] = initialWidth;
                    sizes[2] = new_w;
                }
                ui->splitter->setSizes(sizes);
            }

            if (createNewDocument) {
                QString fileName = editor->getMSAObject()->getDocument()->getURL();
                assert(!fileName.isEmpty());
                if (fileName.isEmpty()) {
                    treeGeneratorTask = NULL;
                    return;
                }

                int sep = QDir::fromNativeSeparators(fileName).lastIndexOf('/');
                int dot = fileName.lastIndexOf('.');
                if (dot != -1 && dot > sep) {
                    fileName = fileName.left(dot);
                }
                QString fileName1 = fileName + ".nwk";
                if (QFile::exists(fileName1) || AppContext::getProject()->findDocumentByURL(fileName1) != NULL) {
                    int i = 0;
                    do {
                        fileName1 = fileName + QString("_%1.nwk").arg(++i);
                    } while (QFile::exists(fileName1) || AppContext::getProject()->findDocumentByURL(fileName1) != NULL);
                }

                PhyTreeObject *newObj = new PhyTreeObject(treeGeneratorTask->getResult(), "Tree");
                DocumentFormat* df = AppContext::getDocumentFormatRegistry()->getFormatById(BaseDocumentFormats::NEWICK);
                IOAdapterFactory *iof = AppContext::getIOAdapterRegistry()->getIOAdapterFactoryById(BaseIOAdapters::LOCAL_FILE);
                Document *d = new Document(df, iof, fileName1, QList<GObject*>()<<newObj);
                AppContext::getProject()->addDocument(d);
                editor->setTree(newObj);
            } else {
                PhyTreeObject *oldObj = editor->getPhyObject();
                QString name = oldObj->getGObjectName();
                PhyTreeObject *newObj = new PhyTreeObject(treeGeneratorTask->getResult(), name);
                Document *d = oldObj->getDocument();
                d->addObject(newObj);
                editor->setTree(newObj);
                d->removeObject(oldObj);
            }
            recomputeView(true);
        }
        treeGeneratorTask = NULL;
    }
}

void MSAEditorTreeViewer::widthChanged(qreal new_width) {
    if (!root || new_width <= 0.0) {
    	return;
    }
    int lWidth = root->getWidth() + (showNameLabels ? maxNameWidth : 0);
    qreal k = (new_width - lWidth) / (root->getWidth() - root->x() - lWidth);
    if (k < 0.00001)
        return;
    int old_width = new_width;

    QStack<GraphicsBranchItem*> stack;
    foreach(QGraphicsItem *item, root->childItems()) {
        GraphicsBranchItem *citem = dynamic_cast<GraphicsBranchItem*>(item);
        if (citem)
            stack.push(citem);
    }
    while (!stack.empty()) {
        new_width = stack.top()->getWidth() * k;
        if (!new_width)
            new_width = k;
        GraphicsBranchItem *top = stack.pop();
        top->setWidth(new_width);

        int w = top->distanceText->boundingRect().width();
        int x = -(new_width + w) / 2;
        top->distanceText->setPos(x, 0);
        if (top->distanceText->scenePos().x() + w > 0) {
            top->distanceText->setPos(-w, 0);
        }

        QList<QGraphicsItem*> items = top->childItems();
        for (int i = 0, s = items.size(); i < s; ++i) {
            GraphicsBranchItem *item = dynamic_cast<GraphicsBranchItem*>(items[i]);
            if (item)
                stack.push(item);
        }
    }
    root->setPos(root->getWidth() - old_width, root->pos().y());
}

void MSAEditorTreeViewer::recomputeView(bool treeChanged) {
    if (root) {
        scene()->removeItem(root);
        delete root;
        root = NULL;
    }
    setSceneRect(0, 0, 0, 1);

    if (treeChanged) {
        updateAligns();
    }

    BranchCreationTask *crt = new BranchCreationTask(editor->getPhyObject()->getTree()->rootNode, editor->getFont(), aligns, editor->getRowHeight());
	connect(crt, SIGNAL(si_calculated(GraphicsBranchItem*)), SLOT(sl_viewRecomputed(GraphicsBranchItem*)));
	TaskScheduler* scheduler = AppContext::getTaskScheduler();
	scheduler->registerTopLevelTask(crt);
}

void MSAEditorTreeViewer::updateAligns() {
    int cur = 0;
    QMap<QString, int> names;
    QStack<PhyNode*> stack;
    stack.push(editor->getPhyObject()->getTree()->rootNode);

    if (editor->getMSAObject() != NULL) {
        const MAlignment &ma = editor->getMSAObject()->getMAlignment();
        int size = ma.getNumSequences();
        for (int i = 0; i < size; ++i) {
            QString str = ma.alignedSeqs[i].name;
            names.insert(str.replace('_', ' '), i);
        }
    }

    maxNameWidth = 0;
    while (!stack.empty()) {
        PhyNode *node = stack.pop();
        int size = node->branches.size();
        for (int i = size - 1; i >= 0; --i) {
            if (node->branches[i]->node2 != node)
                stack.push(node->branches[i]->node2);
        }
        if (size <= 1 && node->name != "" && node->name != "ROOT") {
            if (editor->getMSAObject() != NULL) {
                QString str = node->name.replace('_', ' ');
                assert(names.contains(str));
                aligns.insert(node, names[str]);
            } else {
                aligns.insert(node, cur++);
            }

            QFont font(editor->getFont());
            font.setPointSize(font.pointSize() - 2);
            QFontMetrics fm(font);
            int w = fm.width(node->name) + 2 * GraphicsBranchItem::TEXT_SPACE;
            if (maxNameWidth < w) {
                maxNameWidth = w;
            }
        }
    }
}

void MSAEditorTreeViewer::sl_viewRecomputed(GraphicsBranchItem *gbi) {
    root = gbi;

    root->setVisibleW(false);
    root->setWidthW(GraphicsBranchItem::DEFAULT_WIDTH);
    QPointF pos = root->pos();
    if (ui->seqArea) {
        pos.setY(pos.y() + ui->seqArea->getSequenceYRange(0, true).startPos);
    }
    root->setPos(pos.x() - (showNameLabels ? maxNameWidth : 0) - GraphicsBranchItem::TEXT_SPACE, pos.y());

    scene()->addItem(root);
    int size = ui->splitter->sizes()[0];
    widthChanged(size <= 0 ? initialWidth : size);

	if (editor->getMSAObject() == NULL) {
        int pageStep = parentWidget()->height() / editor->getRowHeight();
        int h = aligns.size() - pageStep + 1;

        oldScrollPos = 0;
        vBar->setMinimum(0);
        vBar->setSingleStep(1);
        vBar->setMaximum(h < 0 ? 0 : h);
        vBar->setPageStep(pageStep);
        vBar->setDisabled(pageStep >= h);
        connect(vBar, SIGNAL(valueChanged(int)), SLOT(sl_onVScrollMoved(int)));
    } else {
        if (parentWidget()->isHidden())
            showTreeAction->trigger();
    }

    showLabels(LabelTypes(LabelType_SequnceName | LabelType_Distance));
    updateToolBar();
}

void MSAEditorTreeViewer::sl_onVScrollMoved(int pos) {
    if (root) {
        root->setPos(root->pos() + QPointF(0, (oldScrollPos - pos) * editor->getRowHeight()));
        oldScrollPos = pos;
    }
}

void MSAEditorTreeViewer::wheelEvent(QWheelEvent *we) {
    vBar->triggerAction(we->delta() > 0 ? QAbstractSlider::SliderSingleStepSub : QAbstractSlider::SliderSingleStepAdd);
    QWidget::wheelEvent(we);
}

void MSAEditorTreeViewer::mousePressEvent(QMouseEvent *e) {
    if (root && !(e->modifiers() & Qt::ShiftModifier))
        root->setSelected(false);
    QGraphicsView::mousePressEvent(e);
}

void MSAEditorTreeViewer::sl_recomputeView() {
    if (root) {
        recomputeView(true);
    }
}

void MSAEditorTreeViewer::sl_startChanged(const QPoint& p, const QPoint& prev) {
    if (root == NULL || ui->seqArea == NULL)
        return;

    int d = ui->seqArea->getSequenceYRange(prev.y(), true).startPos - ui->seqArea->getSequenceYRange(p.y(), true).startPos;
    root->setPos(root->pos() + QPointF(0, d));
}

void MSAEditorTreeViewer::sl_buildStaticToolbar(GObjectView*, QToolBar* tb) {
    if (editor->getMSAObject() != NULL) {
        QMenu *showMenu = new QMenu();
        showTreeAction = showMenu->addAction(tr("show_tree"));
        showTreeAction->setCheckable(true);
        showTreeAction->setChecked(editor->getPhyObject() != NULL);
        connect(showTreeAction, SIGNAL(triggered(bool)), SLOT(sl_showTreeTriggered(bool)));
        showSeqsAction = showMenu->addAction(tr("show_sequences_area"));
        showSeqsAction->setCheckable(true);
        showSeqsAction->setChecked(true);
        connect(showSeqsAction, SIGNAL(triggered(bool)), SLOT(sl_showSeqsTriggered(bool)));

        showButton = new QToolButton();
        QAction *switchAction = new QAction(QIcon(":/core/images/page_refresh.png"), "", showButton);
        showButton->setPopupMode(QToolButton::InstantPopup);
        showButton->setDefaultAction(switchAction);
        showButton->setMenu(showMenu);

        tb->addSeparator();
        buildTreeAction = tb->addAction(QIcon(":/core/images/phylip.png"), tr("build_tree"));
        connect(buildTreeAction, SIGNAL(triggered()), SLOT(sl_buildTreeTriggered()));
        tb->addWidget(showButton);
        sortAction = tb->addAction(QIcon(":/core/images/sort_ascending.png"), tr("sort_tree"));
        connect(sortAction, SIGNAL(triggered()), SLOT(sl_sortTriggered()));
        bool locked = editor->getMSAObject() ? editor->getMSAObject()->isStateLocked() : false;
        sortAction->setEnabled(editor->getPhyObject() != NULL && (!locked));
    }

    QMenu *labelsMenu = new QMenu();
    nameLabelsAction = labelsMenu->addAction(tr("show_name_labels"));
    nameLabelsAction->setCheckable(true);
    nameLabelsAction->setChecked(showNameLabels);
    connect(nameLabelsAction, SIGNAL(triggered(bool)), SLOT(sl_showNameLabelsTriggered(bool)));
    distanceLabelsAction = labelsMenu->addAction(tr("show_distance_labels"));
    distanceLabelsAction->setCheckable(true);
    distanceLabelsAction->setChecked(true);
    connect(distanceLabelsAction, SIGNAL(triggered(bool)), SLOT(sl_showDistanceLabelsTriggered(bool)));
    QToolButton *labelsButton = new QToolButton();
    labelsButton->setPopupMode(QToolButton::InstantPopup);
    labelsAction = new QAction(QIcon(":/core/images/text_ab.png"), "", labelsButton);
    labelsButton->setDefaultAction(labelsAction);
    labelsButton->setMenu(labelsMenu);
    tb->addWidget(labelsButton);

    printAction = tb->addAction(QIcon(":/core/images/printer.png"), tr("print_tree"));
    connect(printAction, SIGNAL(triggered()), SLOT(sl_printTriggered()));
    printAction->setEnabled(editor->getPhyObject() != NULL);

    QMenu *captureMenu = new QMenu();
    captureScreenAction = captureMenu->addAction(tr("capture_screen"));
    connect(captureScreenAction, SIGNAL(triggered()), SLOT(sl_captureScreenTriggered()));
    captureTreeAction = captureMenu->addAction(tr("capture_tree"));
    captureTreeAction->setEnabled(editor->getPhyObject() != NULL);
    connect(captureTreeAction, SIGNAL(triggered()), SLOT(sl_captureTreeTriggered()));
    QToolButton *captureButton = new QToolButton();
    captureButton->setPopupMode(QToolButton::InstantPopup);
    QAction *defaultCaptureAction = new QAction(QIcon(":/core/images/cam2.png"), "", captureButton);
    captureButton->setDefaultAction(defaultCaptureAction);
    captureButton->setMenu(captureMenu);
    tb->addWidget(captureButton);

    updateToolBar();
}

void MSAEditorTreeViewer::fillMAlignment(PhyNode *node, MAlignment &msa) {
    if (editor->getMSAObject() == NULL)
        return;
    for (int i = 0, s = node->branches.size(); i < s; ++i) {
        if (node->branches[i]->node2 != node)
            fillMAlignment(node->branches[i]->node2, msa);
    }
    if (node->name != "" && node->name != "ROOT") {
        const MAlignment &old_msa = editor->getMSAObject()->getMAlignment();
        for (int i = 0, s = old_msa.getNumSequences(); i < s; ++i) {
            QString name = old_msa.alignedSeqs[i].name;
            if (name.replace('_', ' ') == node->name.replace('_', ' ')) {
                msa.alignedSeqs.append(old_msa.alignedSeqs[i]);
                aligns[node] = i;
                break;
            }
        }
    }
}

void MSAEditorTreeViewer::sl_buildTreeTriggered() {
    if (editor->getMSAObject() == NULL)
        return;
    PhyTreeGeneratorRegistry* registry = AppContext::getPhyTreeGeneratorRegistry();
    QStringList list = registry->getNameList();
    if(list.size()==0){
        QMessageBox::information(this, tr("Calculate phy tree"),
            tr("No algorithms for building phylogenetic tree are available.") );
        return;
    }        

    CreatePhyTreeDialogController dlg(editor->getMSAObject()->getMAlignment(), settings);

    int rc = dlg.exec();
    if (rc != QDialog::Accepted)
        return;
    
    if (treeGeneratorTask != NULL && !treeGeneratorTask->isCanceled() && !treeGeneratorTask->hasErrors()) {
        return;
    }
    if (editor->getPhyObject() != NULL) {
        createNewDocument = editor->getMSAObject()->hasObjectRelation(editor->getPhyObject(), MSAEditor::TreeRelation);
    } else {
        createNewDocument = true;
    }

    treeGeneratorTask = new PhyTreeGeneratorTask(editor->getMSAObject()->getMAlignment(), settings);
    connect(treeGeneratorTask, SIGNAL(si_stateChanged()), SLOT(sl_showTree()) );
    TaskScheduler* scheduler = AppContext::getTaskScheduler();
    scheduler->registerTopLevelTask(treeGeneratorTask);
}

void MSAEditorTreeViewer::showLabels(LabelTypes labelTypes) {
    if (root == NULL)
        return;

    QStack<GraphicsBranchItem*> stack;
    stack.push(root);
    while (!stack.isEmpty()) {
        GraphicsBranchItem *node = stack.pop();
        if (node->distanceText != NULL) {
            if (labelTypes.testFlag(LabelType_SequnceName)) {
                if (node->nameText != NULL) {
                    node->nameText->setVisible(showNameLabels);
                }
            }
            if (labelTypes.testFlag(LabelType_Distance)) {
                if (node->distanceText != NULL) {
                    node->distanceText->setVisible(showDistanceLabels);
                }
            }
        }
        foreach (QGraphicsItem* item, node->childItems()) {
            GraphicsBranchItem *bitem = dynamic_cast<GraphicsBranchItem*>(item);
            if (bitem != NULL) {
                stack.push(bitem);
            }
        }
    }
}

void MSAEditorTreeViewer::sl_showNameLabelsTriggered(bool on) {
    int size = ui->splitter->sizes()[0];
    parentWidget()->setMinimumWidth(2 * GraphicsBranchItem::DEFAULT_WIDTH + (on ? maxNameWidth : 0));
    widthChanged((size <= 0 ? initialWidth : size) + (on ? -1 : 1) * maxNameWidth);
    QPointF pos = root->pos();
    root->setPos(pos.x() + (on ? -1 : 1) * maxNameWidth, pos.y());
    showNameLabels = on;
    showLabels(LabelType_SequnceName);
}

void MSAEditorTreeViewer::sl_showDistanceLabelsTriggered(bool on) {
    showDistanceLabels = on;
    showLabels(LabelType_Distance);
}

void MSAEditorTreeViewer::sl_showTreeTriggered(bool on) {
    showSeqsAction->setEnabled(on);
    if (on) {
        parentWidget()->show();
        ui->splitter->restoreState(state);
    } else {
        state = ui->splitter->saveState();
        parentWidget()->hide();
    }
}

void MSAEditorTreeViewer::sl_showSeqsTriggered(bool on) {
    showTreeAction->setEnabled(on);
    if (on) {
        ui->splitter->widget(1)->show();
        ui->splitter->widget(2)->show();
        ui->splitter->restoreState(state);
    } else {
        state = ui->splitter->saveState();
        ui->splitter->widget(1)->hide();
        ui->splitter->widget(2)->hide();
    }
}

void MSAEditorTreeViewer::sl_printTriggered() {
    QPrinter printer;
    QPrintDialog dialog(&printer, this);
    if (dialog.exec() != QDialog::Accepted)
        return;

    QPainter painter(&printer);
    QRect rect = viewport()->rect();
    QSize size = rect.size();
    QRect pageRect = printer.pageRect();
    qreal kx = size.width() * (1.0 / pageRect.width());
    qreal ky = size.height() * (1.0 / pageRect.height());
    if (kx > 1 || ky > 1)
        paint(painter, rect, kx > ky ? kx : ky);
    else
        paint(painter, rect);
}

void MSAEditorTreeViewer::sl_captureTreeTriggered() {
    QString fileName, format;
    saveImageDialog(fileName, format);
    if (!fileName.isEmpty()) {
        QRect rect = viewport()->rect();
        QImage image(rect.size(), QImage::Format_RGB32);
        image.fill(0xFFFFFFFF);
        QPainter painter(&image);
        paint(painter, rect);
        image.save(fileName, format.toAscii().constData());
    }
}

void MSAEditorTreeViewer::paint(QPainter &painter, QRectF rect, qreal k) {
    painter.setBrush(QColor(0, 0, 0));
    painter.setFont(editor->getFont());
    if (k > -1)
        painter.setTransform(QTransform::fromScale(1/k, 1/k));
    render(&painter, rect, getSceneBoundingRect());
}

QRect MSAEditorTreeViewer::getSceneBoundingRect() {
    if (editor->getMSAObject() == NULL)
        return QRect(0, 0, 0, 0);
    int xpos = root->pos().x() - GraphicsBranchItem::DEFAULT_WIDTH;
    int ypos = ui->seqArea->getSequenceYRange(0, true).startPos;
    int h = ui->seqArea->getSequenceYRange(ui->seqArea->getFirstVisibleSequence() +
        editor->getMSAObject()->getMAlignment().getNumSequences() - 1, true).endPos();
    return QRect(0, ypos, -xpos, h);
}

void MSAEditorTreeViewer::saveImageDialog(QString &fileName, QString &format) {
    QString filters =
        "PNG - Portable Network Graphics (*.png);;"
        "BMP - Windows Bitmap (*.bmp);;"
        "GIF - Graphic Interchange Format (*.gif);;"
        "JPG/JPEG format (*.jpg);;"
        "PBM - Portable Bitmap (*.pbm);;"
        "PPM - Portable Pixmap (*.ppm);;"
        "TIFF - Tagged Image File format (*.tif);;"
        "XBM - X11 Bitmap (*.xbm);;"
        "XPM - X11 Pixmap (*.xpm)";

    LastOpenDirHelper lod(IMAGE_DIR);
    Document *d = editor->getPhyObject() ? editor->getPhyObject()->getDocument() : editor->getMSAObject()->getDocument();
    QString defaultName = d->getName();
    int i = defaultName.lastIndexOf('.');
    if (i != -1) {
        defaultName = defaultName.left(i);
    }
    QString initialPath = lod.dir + "/" + defaultName;
    QString selectedFormat;
    fileName = QFileDialog::getSaveFileName(this, tr("Save As"),
        initialPath, filters, &selectedFormat);
    lod.url = fileName;
    if (fileName.isEmpty())
        return;

    format = selectedFormat.left(3).toLower();
    if (!fileName.endsWith("." + format)) {
        fileName.append("." + format);
        if (QFile::exists(fileName)) {
            QMessageBox::StandardButtons b = QMessageBox::warning(0, tr("Replace file"),
                tr("%1 already exists.\nDo you want to replace it?").arg(fileName),
                QMessageBox::Yes | QMessageBox::No);
            if (QMessageBox::Yes != b) {
                return;
            }
        }
    }
}

void MSAEditorTreeViewer::sl_captureScreenTriggered() {
    QString fileName, format;
    saveImageDialog(fileName, format);
    if (!fileName.isEmpty())
        ui->saveScreenshot(fileName, format.toAscii().constData());
}

void MSAEditorTreeViewer::sl_sortTriggered() {
    if (!root || editor->getMSAObject() == NULL)
        return;

    MAlignment msa;
    msa.alphabet = editor->getMSAObject()->getMAlignment().alphabet;
    fillMAlignment(editor->getPhyObject()->getTree()->rootNode, msa);
    editor->getMSAObject()->setMAlignment(msa);
    updateAligns();
    recomputeView(false);
}

MSAEditorTreeViewer::~MSAEditorTreeViewer() {
    delete scene();
}

void MSAEditorTreeViewer::sl_alignmentChanged( const MAlignment& beforMA, const MAlignmentModInfo& mi) {
    if (!mi.sequenceListChanged) {
        return;
    }

    QSet<QString> names;
    for (int i = 0, s = editor->getMSAObject()->getMAlignment().getNumSequences(); i < s; ++i) {
        QString str = editor->getMSAObject()->getMAlignment().alignedSeqs[i].name;
        names.insert(str.replace('_', ' '));
    }

    QList<Document*> docs = AppContext::getProject()->getDocuments();
    foreach (Document* d, docs) {
        QList<GObject*> objs = d->getObjects();
        foreach (GObject* obj, objs) {
            if (obj->getGObjectType() == GObjectTypes::PHYLOGENETIC_TREE) {
                PhyTreeObject *phyObj = qobject_cast<PhyTreeObject*>(obj);
                if (PhyTreeUtils::getNumSeqsFromNode(phyObj->getTree()->rootNode, names) == editor->getMSAObject()->getMAlignment().getNumSequences()) {
                    phyObj->addObjectRelation(editor->getMSAObject(), MSAEditor::MSARelation);
                    editor->getMSAObject()->addObjectRelation(phyObj, MSAEditor::TreeRelation);
                } else {
                    phyObj->removeObjectRelation(GObjectRelation(editor->getMSAObject(), MSAEditor::MSARelation));
                    editor->getMSAObject()->removeObjectRelation(GObjectRelation(phyObj, MSAEditor::TreeRelation));
                }
            }
        }
    }

    updateToolBar();
}

void MSAEditorTreeViewer::setTree(PhyTreeObject* obj) {
    if (obj->hasObjectRelation(editor->getMSAObject(), MSAEditor::MSARelation)) {
        recomputeView(true);
    }
}

void MSAEditorTreeViewer::resizeEvent(QResizeEvent *e) {
    if (e->size().width() > 0) {
        widthChanged(e->size().width());
    }
    QGraphicsView::resizeEvent(e);
}

void MSAEditorTreeViewer::updateToolBar() {
    bool hasObj = editor->getPhyObject() != NULL;
    if (editor->getMSAObject() != NULL) {
        bool hasRel = hasObj && editor->getMSAObject()->hasObjectRelation(editor->getPhyObject(), MSAEditor::TreeRelation);
        sortAction->setEnabled(!editor->getMSAObject()->isStateLocked() && hasRel);
        showButton->setEnabled(hasObj);
        printAction->setEnabled(hasRel);
        captureTreeAction->setEnabled(hasRel);
        captureScreenAction->setEnabled(true);
    } else {
        printAction->setEnabled(hasObj);
        captureTreeAction->setEnabled(hasObj);
        captureScreenAction->setEnabled(false);
    }
    nameLabelsAction->setEnabled(hasObj);
    distanceLabelsAction->setEnabled(hasObj);
    labelsAction->setEnabled(hasObj);
}

}//namespace
