/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "ProjectTasks.h"

#include "ProjectServiceImpl.h"
#include "ProjectImpl.h"
#include "ProjectLoaderImpl.h"
#include <core_services/AppContextImpl.h>

#include <util_tasks/SaveDocumentTask.h>
#include <util_tasks/AddDocumentTask.h>
#include <util_tasks/LoadDocumentTask.h>
#include <util_tasks/RemoveDocumentTask.h>

#include <document_format/DocumentFormatUtils.h>

#include <core_api/IOAdapter.h>
#include <core_api/Log.h>
#include <core_api/ObjectViewModel.h>
#include <core_api/DocumentModel.h>
#include <core_api/DocumentFormats.h>
#include <core_api/GHints.h>
#include <core_api/ProjectView.h>
#include <core_api/GObject.h>
#include <core_api/AppSettings.h>
#include <core_api/L10n.h>

#include <gobjects/UnloadedObject.h>
#include <gobjects/GObjectUtils.h>

#include <QtXml/QDomDocument>
#include <QtGui/QMessageBox>

#include <QtCore/QMutex>

namespace GB2 {

static LogCategory log(ULOG_CAT_CORE_SERVICES);

//////////////////////////////////////////////////////////////////////////
///Close project
CloseProjectTask::CloseProjectTask() : Task(tr("close_project_task_name"), TaskFlags_NR_FOSCOE)
{
}

void CloseProjectTask::prepare() {
	if (AppContext::getProject()==NULL) {
		stateInfo.setError(  tr("error_no_active_project") );
		return;
	}
	/* TODO: this is done by project view. Need to cleanup this part! 
	addSubTask(new SaveProjectTask(SaveProjectTaskKind_SaveProjectAndDocumentsAskEach));
	*/
	QList<Task*> tasks;
	/**/
	Project *pp = AppContext::getProject();
    if (pp->isTreeItemModified()) {
		tasks.append(AppContext::getProjectService()->saveProjectTask(SaveProjectTaskKind_SaveProjectAndDocumentsAskEach));
	}
	/**/
	ServiceRegistry* sr = AppContext::getServiceRegistry();
	QList<Service*> services = sr->findServices(Service_Project);
	assert(services.size() == 1);
	Service* projectService = services.first();
	tasks.append(sr->unregisterServiceTask(projectService));
	addSubTask(new MultiTask(tr("Save and close project"), tasks));
}


//////////////////////////////////////////////////////////////////////////
/// OpenProjectTask
OpenProjectTask::OpenProjectTask(const QString& _url, bool _closeActiveProject, const QString& /*_name*/) 
	: Task(tr("open_project_task_name"), TaskFlags_NR_FOSCOE), url(_url), loadProjectTask(NULL), closeActiveProject(_closeActiveProject)
{
}

OpenProjectTask::OpenProjectTask(const QList<QUrl>& list, bool _closeActiveProject) 
	: Task(tr("open_project_task_name"), TaskFlags_NR_FOSCOE), urlList(list), loadProjectTask(NULL), closeActiveProject(_closeActiveProject) 
{
}



void OpenProjectTask::prepare() {
	if (urlList.size() == 1 && url.isEmpty()) {
		//FIXME issue 534
		QUrl tmp = urlList.takeFirst();
		if (tmp.isRelative() || tmp.scheme() == "file") {
			url = tmp.toLocalFile();
		} else {
			url = tmp.toString();
		}
	}

	if (url.endsWith(PROJECTFILE_EXT)) { // open a project
		QFileInfo f(url);
		if (f.exists() && !(f.isFile() && f.isReadable())) {
			stateInfo.setError(  tr("invalid_url%1").arg(url) );
			return;
		}

		// close current
		if (AppContext::getProject()!=NULL) {
			if (!closeActiveProject) {
				stateInfo.cancelFlag = true;
				log.info(tr("Stopped loading project: %1. Reason: active project found").arg(url));
				return;
			}
			addSubTask(new CloseProjectTask());
		}

		if (f.exists()) {
			loadProjectTask = new LoadProjectTask(url);
			addSubTask(loadProjectTask);
		} else {
			ProjectImpl* p =  new ProjectImpl(name, url);
			addSubTask(new SaveProjectTask(SaveProjectTaskKind_SaveProjectOnly, p));
			addSubTask(new RegisterProjectServiceTask(p));
		}
	} else { // load a (bunch of) documents
		if (!url.isEmpty()) {
			urlList << BaseIOAdapters::str2url(url);
		}
		Project* p = AppContext::getProject();
		if (!p) {
			// create anonymous project
			log.info(tr("Creating new project"));
			p = new ProjectImpl("", "");
			Task* rpt = new RegisterProjectServiceTask(p);
			rpt->setSubtaskProgressWeight(0);
			addSubTask(rpt);
		}
		foreach(QUrl _url, urlList) {
			//FIXME issue 534
			QString url = _url.toString();
			if (_url.isRelative() || _url.scheme() == "file") {
				QFileInfo fi(_url.toLocalFile());
				url = fi.canonicalFilePath();
			}
			if (url.endsWith(PROJECTFILE_EXT)) {
				// skip extra project files
				stateInfo.setError(  tr("ignore %1").arg(url) );
				continue;
			}
			Document * doc = p->findDocumentByURL(url);
			if (!doc) {
				QList<DocumentFormat*> fs = DocumentFormatUtils::detectFormat(url);
				if (fs.isEmpty()) {
					stateInfo.setError(  tr("unsupported_document%1").arg(url) );
					continue;
				}

				DocumentFormat* format = fs.first();
				assert(format);
				IOAdapterFactory* iof = AppContext::getIOAdapterRegistry()->getIOAdapterFactoryById(BaseIOAdapters::url2io(url));
				doc = new Document(format, iof, url);
				Task * adt = new AddDocumentTask(doc);
				adt->setSubtaskProgressWeight(0);
				addSubTask(adt);
			} else{
				QWidget *p = AppContextImpl::getMainWindow()->getQMainWindow();
				log.details("Selected document already in project");
				QMessageBox::warning(p, tr("warning"), tr("Selected document already in project"));
				if (doc->isLoaded()) {
					const QList<GObject*>& docObjects = doc->getObjects();
					QList<GObjectViewWindow*> viewsList = GObjectViewUtils::findViewsWithAnyOfObjects(docObjects);
					if (!viewsList.isEmpty()) {
						AppContext::getMainWindow()->getMDIManager()->activateWindow(viewsList.first());
					}else{
						AppContext::getProjectView()->highlightItem(doc);
					}
					log.info(tr("Document is already opened: %1").arg(url));
					continue;
				}else if(!doc->isLoaded() && AppContext::getProjectView()){
					AppContext::getProjectView()->highlightItem(doc);
					continue;
				}
			}

			if (getSubtasks().size() < 50) {
				addSubTask(new LoadUnloadedDocumentAndOpenViewTask(doc));
			}
		}
	}
}

QList<Task*> OpenProjectTask::onSubTaskFinished(Task* subTask) {
	QList<Task*> res;
	if (!isCanceled() && subTask == loadProjectTask && !loadProjectTask->hasErrors()) {
		Project* p =  loadProjectTask->detachProject();
		res.append(new RegisterProjectServiceTask(p));
	}
	return res;
}

//////////////////////////////////////////////////////////////////////////
/// Save project
SaveProjectTask::SaveProjectTask(SaveProjectTaskKind _k, Project* p, const QString& _url) 
	: Task(tr("save_project_task_name"), TaskFlag_NoRun), k(_k), proj(p), url(_url)
{
}

SaveProjectTask::~SaveProjectTask () {
}

void SaveProjectTask::prepare() {
	if (proj == NULL) {
		proj = AppContext::getProject();
	}
	assert(proj!=NULL);
	if (url.isEmpty()) {
		url = proj->getProjectURL();
	}
	QList<Task *> ssTasks;
	if (url.isEmpty() && (!proj->getGObjectViewStates().isEmpty() || proj->getDocuments().size() > 1)) {
		//ask if to save project
		int code = QMessageBox::question(NULL, tr("question"), tr("save_project_file_question"), QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
		if (code == QMessageBox::Yes) {
			ProjectDialogController d(ProjectDialogController::Save_Project);
			d.projectFolderEdit->setText(QDir::home().absolutePath());
			d.projectNameEdit->setText(tr("My project"));
			int rc = d.exec();
			if (rc == QDialog::Accepted) {
				AppContext::getProject()->setProjectName(d.projectNameEdit->text());
				url = d.projectFolderEdit->text() + "/" + d.projectFileEdit->text();
				if (!url.endsWith(PROJECTFILE_EXT)) {
					url.append(PROJECTFILE_EXT);
				}
				AppContext::getProject()->setProjectURL(url);
			}
		}
	}

	if (k!=SaveProjectTaskKind_SaveProjectOnly) {
		QList<Document*> modifiedDocs = SaveMiltipleDocuments::findModifiedDocuments(AppContext::getProject()->getDocuments());
		if (!modifiedDocs.isEmpty()) {
			ssTasks.append(new SaveMiltipleDocuments(modifiedDocs, k == SaveProjectTaskKind_SaveProjectAndDocumentsAskEach));				
		}
	}
	ssTasks.append(new SaveOnlyProjectTask(proj, url));
	addSubTask(new MultiTask(tr("Save documents, remove phantom docs, save  project"), ssTasks));
}

Task::ReportResult SaveProjectTask::report() {
	return Task::ReportResult_Finished;
}


static QString map2String(const QVariantMap& map) {
	QByteArray a;
	QVariant v(map);
	QDataStream s(&a, QIODevice::WriteOnly);
	s << v;
	QString res(a.toBase64());
	return res;
}


//////////////////////////////////////////////////////////////////////////
/// SaveOnlyProjectTask

SaveOnlyProjectTask::SaveOnlyProjectTask(Project* p, const QString& _url)
	:Task(tr("Save project"), TaskFlag_NoRun), proj(p), url(_url){
		lock = NULL;
		//assert(!url.isEmpty());
}

SaveOnlyProjectTask::~SaveOnlyProjectTask(){
	assert(lock == NULL);
}

void SaveOnlyProjectTask::prepare(){
	if (proj == NULL) {
		proj = AppContext::getProject();
	}
	assert(proj!=NULL);
	if (url.isEmpty()) {
		url = proj->getProjectURL();
	}

	foreach(Document *d, proj->getDocuments()){
		IOAdapterFactory *IOAF = d->getIOAdapterFactory();
		IOAdapterId id = IOAF->getAdapterId();
		if(id == BaseIOAdapters::LOCAL_FILE || id == BaseIOAdapters::GZIPPED_LOCAL_FILE){
			QFile pathToDoc(d->getURL());
			if(!pathToDoc.exists()){
				phantomDocs.append(d);
			}
		}
	}
	if(!phantomDocs.isEmpty()){   
		sub = new RemoveMultipleDocumentsTask(proj, phantomDocs, false, false);
		addSubTask(sub);
	}else{
		_run();
	}
}

QList<Task*> SaveOnlyProjectTask::onSubTaskFinished(Task* subTask){
	QList<Task*> ret;
	if (!isCanceled() && subTask == sub){
		_run();
	}
	return ret;
}

void SaveOnlyProjectTask::_run(){
	lock = new StateLock(getTaskName(), StateLockFlag_LiveLock);
	proj->lockState(lock);

	log.info(tr("Saving project %1").arg(url));
	saveProjectFile(stateInfo, proj, url);
}

Task::ReportResult SaveOnlyProjectTask::report(){
	if (!stateInfo.hasErrors() && url == proj->getProjectURL()) {
		proj->setModified(false);
	}
	proj->unlockState(lock);
	delete lock;
	lock = NULL;
	return Task::ReportResult_Finished;
}

static QVariant toAbsoluteRelations(const QList<GObjectRelation>& relList, const QDir& projDir)  {
    QList<GObjectRelation> absoluteRelations;
    foreach(GObjectRelation rel, relList){
        QFileInfo ff(rel.ref.docUrl);
        if(!ff.isAbsolute()){
            QFileInfo tmpFi(projDir.path() + "/" + rel.ref.docUrl);
            rel.ref.docUrl = tmpFi.canonicalFilePath();
        }
        absoluteRelations.append(rel);
    }
    return QVariant::fromValue<QList<GObjectRelation> >(absoluteRelations);
}

static QVariant toRelativeRelations(const QList<GObjectRelation>& absRelations, const QDir& projDir, 
                                    const QMap<QString, QString>& urlRemap = QMap<QString, QString>()) 
{
    QList<GObjectRelation> relativeRelations;
    foreach (const GObjectRelation& absRel, absRelations) {
        QString absUrl = absRel.ref.docUrl;
        if (urlRemap.contains(absUrl)) {
            absUrl = urlRemap.value(absUrl);
        }
        QFileInfo absFi(absUrl);
        QString relativeURL = projDir.relativeFilePath(absFi.canonicalFilePath());
        GObjectRelation relRel = absRel;
        relRel.ref.docUrl = relativeURL;
        relativeRelations.append(relRel);
    }
    return QVariant::fromValue<QList<GObjectRelation> >(relativeRelations);
}

void SaveOnlyProjectTask::saveProjectFile(TaskStateInfo& ts, Project* project, 
                                          const QString& projectUrl, const QMap<QString, QString>& docUrlRemap) 
{
	QDomDocument xmlDoc("GB2PROJECT");

	QDomElement projectElement = xmlDoc.createElement("gb2project");
	projectElement.setAttribute("name", project->getProjectName());
	const QString ver(CURRENT_PROJECT_VERSION);
	projectElement.setAttribute("version", ver);

    QFileInfo projectFile(projectUrl);
    QDir projectDir = projectFile.absoluteDir();

	//save documents
	foreach(Document* gbDoc, project->getDocuments()) {
		gbDoc->getDocumentFormat()->updateFormatSettings(gbDoc);

        QString docUrl = gbDoc->getURL();
        if (docUrlRemap.contains(docUrl)) {
            docUrl = docUrlRemap.value(docUrl);
        }

		QString adapterId = gbDoc->getIOAdapterFactory()->getAdapterId();
		QDomElement docElement = xmlDoc.createElement("document");
		docElement.setAttribute("io-adapter", adapterId);

        //for local file save relative path
		if (adapterId == BaseIOAdapters::LOCAL_FILE || adapterId == BaseIOAdapters::GZIPPED_LOCAL_FILE) {
            QString relativeUrl = projectDir.relativeFilePath(docUrl);
			docElement.setAttribute("url", relativeUrl);
		} else {
			docElement.setAttribute("url", docUrl);
		}
		
		//store lock info for documents
        DocumentFormat* f = gbDoc->getDocumentFormat();
		QString formatId = f->getFormatId();
		docElement.setAttribute("format", formatId);
		docElement.setAttribute("readonly", gbDoc->hasUserModLock() ? 1 : 0);
		StateLock* l = gbDoc->getDocumentModLock(DocumentModLock_FORMAT_AS_INSTANCE);
		if (l!=NULL) {
			docElement.setAttribute("format-lock", 1);
		}

        //store doc-level hints
		if (!gbDoc->getGHintsMap().isEmpty()) {
		    QString hintsStr = map2String(gbDoc->getGHintsMap());
		    QDomText hintsNode = xmlDoc.createCDATASection(hintsStr);
		    docElement.appendChild(hintsNode);
        }

		//now save unloaded objects info for all document objects 
        foreach(GObject* obj, gbDoc->getObjects()) {
            QDomElement objElement = xmlDoc.createElement("object");
            UnloadedObjectInfo info(obj);
            objElement.setAttribute("name", info.name);
            objElement.setAttribute("type", info.type);
            if (!info.hints.isEmpty()) {
                //for all object relations make path relative
                info.hints[RELATED_OBJECTS_KEY] = toRelativeRelations(obj->getObjectRelations(), projectDir, docUrlRemap);
                QString hintsStr = map2String(info.hints);
                QDomText hintsNode = xmlDoc.createCDATASection(hintsStr);
                objElement.appendChild(hintsNode);
            }
            docElement.appendChild(objElement);
        }
		projectElement.appendChild(docElement);
	}

	//save views states
	foreach(GObjectViewState* view, project->getGObjectViewStates()) {
		//save document info
		QDomElement viewElement = xmlDoc.createElement("view");
		viewElement.setAttribute("factory", view->getViewFactoryId());
		viewElement.setAttribute("viewName", view->getViewName());
		viewElement.setAttribute("stateName", view->getStateName());
		QString dataStr = map2String(view->getStateData());
		QDomText textNode = xmlDoc.createCDATASection(dataStr);
		viewElement.appendChild(textNode);

		projectElement.appendChild(viewElement);
	}

	xmlDoc.appendChild(projectElement);

	QByteArray rawData = xmlDoc.toByteArray();
	//	printf(">>%s", xmlDoc.toString().toStdString().c_str());

    //check that project directory exists
    if (!projectDir.exists()) {
        QDir root;
        root.mkpath(projectDir.absolutePath());
    }

    //save project file text to file
    QFile f(projectUrl);
	if (f.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
		qint64 s = f.write(rawData);
		f.close();
		if (s!=rawData.size()) {
			ts.setError(L10N::errorWritingFile(projectUrl));
		}
	} else {
		ts.setError(L10N::errorOpeningFileWrite(projectUrl));
	}
}

//////////////////////////////////////////////////////////////////////////
/// LoadProjectTask
LoadProjectTask::LoadProjectTask(const QString& _url) 
	: Task(tr("load_project_task_name"), TaskFlag_None), proj(NULL), url(_url)
{
	xmlDoc = new QDomDocument();
}

LoadProjectTask::~LoadProjectTask() {
	if (proj) {
		delete proj;
	}
	delete xmlDoc;
}

void LoadProjectTask::run() {
	log.details(tr("Loading project from: %1").arg(url));
	loadXMLProjectModel(url, stateInfo, *xmlDoc, version);
}

Task::ReportResult LoadProjectTask::report() {
	if (!stateInfo.hasErrors()) {
		ProjectParserRegistry *ppr = ProjectParserRegistry::instance();
		ProjectParser *parser = ppr->getProjectParserByVersion(version);
		if(parser == NULL){
			log.info(tr("Unable to detect parser for project"));
		}else{
			proj = parser->createProjectFromXMLModel(url, *xmlDoc, stateInfo);
		}
		if (proj!=NULL) {
			log.info(tr("Project loaded: %1").arg(url));
		}
	}
	return Task::ReportResult_Finished;
}


static QVariantMap string2Map(const QString& string, bool emptyMapIfError) {
	Q_UNUSED(emptyMapIfError);

	QDataStream s(QByteArray::fromBase64(string.toAscii()));
	QVariant res(QVariant::Map);
	s >> res;
	if (res.type() == QVariant::Map) {
		return res.toMap();
	}
	assert(emptyMapIfError);
	return QVariantMap();
}

void LoadProjectTask::loadXMLProjectModel(const QString& url, TaskStateInfo& si, QDomDocument& doc, ProjectVersionId &ver) {
	assert(doc.isNull());

	QFile f(url);
	if (!f.open(QIODevice::ReadOnly)) {
		si.setError(  tr("error_open_file %1").arg(url) );
		return;
	}
	QByteArray  xmlData = f.readAll();
	f.close();

	bool res = doc.setContent(xmlData);

	const QString curVerStr(CURRENT_PROJECT_VERSION);

	QDomElement projectElement = doc.documentElement();
	const QString projVerStr = projectElement.attribute("version");

	if(!projVerStr.isEmpty()){
		bool ok;
		double projVer = projVerStr.toDouble(&ok), curVer = curVerStr.toDouble();

		if (!ok) {
			si.setError(tr("Project version is not a double value"));
			doc.clear();
		}

		if (projVer > curVer){
			si.setError(tr("This document was created by newer version of UGENE. Please update UGENE."));
			doc.clear();
		}
		ver = projVerStr;
	} else {
		ver = "1.0";
	}

	if (!res) {
		si.setError(  tr("error_invalid_content %1").arg(url) );
		doc.clear();
	}
	if (doc.doctype().name()!="GB2PROJECT") {
		si.setError(  tr("error_invalid_content %1").arg(url) );
		doc.clear();
	}
}

RegisterProjectServiceTask::RegisterProjectServiceTask(Project* _proj) 
	: Task(tr("Register project"), TaskFlag_NoRun), proj(_proj)
{
}

void RegisterProjectServiceTask::prepare() {
	ProjectServiceImpl* ps = new ProjectServiceImpl(proj);
	addSubTask(AppContext::getServiceRegistry()->registerServiceTask(ps));
}

OpenProjectTaskAndExecDialog::OpenProjectTaskAndExecDialog(QDialog *_dialog)
	:OpenProjectTask(QString(), true)
	, dialog(_dialog){

};

Task::ReportResult OpenProjectTaskAndExecDialog::report(){
	dialog->exec();
	delete dialog;
	return ReportResult_Finished;
}


void ProjectParserRegistry::init(){
	parsers.qlist.append(new ProjectParser10());
}


ProjectParser* ProjectParserRegistry::getProjectParserByVersion( ProjectVersionId id ){
	foreach(ProjectParser *p,  parsers.qlist){
		if(p->getVersion() == id){
			return p;
		}
	}
	return NULL;
}

ProjectParserRegistry * ProjectParserRegistry::instance(){
	static ProjectParserRegistry ppr;
	return &ppr;
}


Project* ProjectParser10::createProjectFromXMLModel( const QString& pURL, const QDomDocument& xmlDoc, TaskStateInfo& si ){
	Q_UNUSED(si); //TODO: report about errors using si!!!

	QDomElement projectElement = xmlDoc.documentElement();
	QString name = projectElement.attribute("name");

	ProjectImpl* pi = new ProjectImpl(name, pURL);

	//read all documents
	QDomNodeList documentList = projectElement.elementsByTagName("document");
	for(int i=0; i < documentList.size(); i++) {
		QDomNode dn = documentList.item(i);
		if (!dn.isElement()) {
			continue;
		}
		QDomElement docElement = dn.toElement();
		QString ioAdapterId = docElement.attribute("io-adapter");
		QFileInfo proj(pURL);
		//QString docURL = proj.absoluteDir().path() + "/" + docElement.attribute("url");
		QString fURL(docElement.attribute("url"));
		QFileInfo fi(docElement.attribute("url"));
		QString docURL = docElement.attribute("url");
		if(ioAdapterId == BaseIOAdapters::GZIPPED_LOCAL_FILE || ioAdapterId == BaseIOAdapters::LOCAL_FILE){
			if(!fi.isAbsolute()){
                QString tmp = proj.absoluteDir().path() + "/" + docElement.attribute("url");
                QFileInfo ff(proj.absoluteDir().path() + "/" + docElement.attribute("url"));
                docURL = ff.canonicalFilePath();
			}else{
				docURL = fi.canonicalFilePath();
			}
		}
		DocumentFormatId format = docElement.attribute("format");
		bool readonly = docElement.attribute("readonly").toInt() != 0;
		bool instanceLock = docElement.attribute("format-lock").toInt() != 0;
		IOAdapterFactory* iof = AppContext::getIOAdapterRegistry()->getIOAdapterFactoryById(ioAdapterId);
		DocumentFormat* df = AppContext::getDocumentFormatRegistry()->getFormatById(format);
		assert(df!=NULL);
		QVariantMap fs = string2Map(docElement.text(), true);

		QList<UnloadedObjectInfo> unloadedObjects;
		QDomNodeList objectList = docElement.elementsByTagName("object");
		QSet<QString> objNames;
		for(int j=0; j < objectList.size(); j++) {
			QDomNode on = objectList.item(j);
			if (!on.isElement()) {
				continue;
			}
			QDomElement objElement = on.toElement();
			UnloadedObjectInfo info;
			info.name = objElement.attribute("name");
			info.type = objElement.attribute("type");
			info.hints = string2Map(objElement.text(), true);

			if (!info.hints.isEmpty()) {
                QVariant qv = info.hints[RELATED_OBJECTS_KEY];
                if (!qv.isNull()){
                   QList<GObjectRelation> relList = qv.value<QList<GObjectRelation> >();
                   info.hints[RELATED_OBJECTS_KEY] = toAbsoluteRelations(relList, proj.absoluteDir());
				}
			}

			if (info.isValid() && !objNames.contains(info.name)) {
				unloadedObjects.append(info);
				objNames.insert(info.name);
			}
		}
		Document* d = new Document(df, iof, docURL, unloadedObjects, fs, instanceLock ? tr("last_loaded_state_was_locked_by_format") : QString());
		d->setUserModLock(readonly);
		pi->addDocument(d);
	}

	// read all saved views
	QDomNodeList viewElements = projectElement.elementsByTagName("view");
	for(int i=0;i<viewElements.size(); i++) {
		QDomNode n = viewElements.item(i);
		assert(n.isElement());
		if (!n.isElement()) {
			continue;
		}
		QDomElement viewElement = n.toElement();
		GObjectViewFactoryId id = viewElement.attribute("factory");
		QString viewName = viewElement.attribute("viewName");
		QString stateName = viewElement.attribute("stateName");
		QVariantMap map  = string2Map(viewElement.text(), false);
		GObjectViewState* state = new GObjectViewState(id, viewName, stateName, map);
		pi->addState(state);
	}
	pi->setModified(false);
	return pi;
}


ExportProjectTask::ExportProjectTask( QString _destinationDir, bool _compress ):
Task(tr("Export project task"), TaskFlags_NR_FOSCOE), compress(_compress), destinationDir(_destinationDir){

}

void ExportProjectTask::prepare(){
	Project *pr = AppContext::getProject();

	QList<Document*> docList = pr->getDocuments();
	foreach(Document* doc, docList){
		if(doc->getURL().isEmpty()){
			log.error(tr("at least one of documents have empty URL"));
			return;
		}
	}

	QDir dDir(destinationDir);
	if(!dDir.exists()){
		log.error(tr("given directory %1 is not exists").arg(dDir.absolutePath()));
		return;
	}
	QMap<QString, QString> urlRemap;

	foreach(Document* doc, docList){
		QString origPath = doc->getURL();
		IOAdapterId id = doc->getIOAdapterFactory()->getAdapterId();
		if (id == BaseIOAdapters::LOCAL_FILE || id == BaseIOAdapters::GZIPPED_LOCAL_FILE){
			QFile f(origPath);
			QFileInfo fi(f);
			QString resultPath = destinationDir + "/" + fi.fileName();
			if(!f.copy(resultPath)){
				log.error(tr("Error during coping documents"));
				return;
			}
            urlRemap[origPath] = resultPath;
		}
	}

	QFileInfo prFi(pr->getProjectURL());
    SaveOnlyProjectTask::saveProjectFile(stateInfo, pr, destinationDir + "/" + prFi.fileName(), urlRemap);
}

Task::ReportResult ExportProjectTask::report(){
	return ReportResult_Finished;
}


void GTest_LoadProject::init(XMLTestFormat* tf, const QDomElement& el) {
	Q_UNUSED(tf);

	TaskResourceUsage tru( RESOURCE_PROJECT, 1, true);
	taskResources.append(tru);
	projContextName = el.attribute("index");
	if(!el.attribute("load_from_temp").isEmpty()){
		url = env->getVar("TEMP_DATA_DIR") + "/" + el.attribute("url");
	}else{
		url = env->getVar("COMMON_DATA_DIR") + "/" + el.attribute("url");
	}
}

void GTest_LoadProject::prepare(){
	QList<Task*> tasks;
	Project *previousProject = AppContext::getProject();
	Task *tt;
	ServiceRegistry* sr = AppContext::getServiceRegistry();
	QList<Service*> services = sr->findServices(Service_Project);
	int servSize = services.size();
	assert(servSize <= 1);
	if(servSize == 1){
		if(previousProject->metaObject() != NULL){
			tt  = new CloseProjectTask();
			tasks.append(tt);
		}
	}else if(previousProject != NULL){
		AppContextImpl::getApplicationContext()->setProject(NULL);
	}
	contextAdded = false;

	loadTask = new LoadProjectTask(url);
	tasks.append(loadTask);
	mt = new MultiTask(tr("Unload previous project, then load new"), tasks);
	addSubTask(mt);
}

Task::ReportResult GTest_LoadProject::report() {
    if (loadTask!=NULL && loadTask->hasErrors()) {
		stateInfo.setError( loadTask->getError());
	} else if (!projContextName.isEmpty()) {
		addContext(projContextName, loadTask->getProject());
		contextAdded = true;
	}
	return ReportResult_Finished;
}

QList<Task*> GTest_LoadProject::onSubTaskFinished( Task* subTask ){
	QList<Task*> subTasks;
	if (subTask->hasErrors()) {
		return subTasks;
	}
	if (subTask == mt) {
		Project *p = loadTask->getProject();
		if(p != NULL){
			AppContextImpl::getApplicationContext()->setProject(p);
		}
	}
	return subTasks;
}

void GTest_LoadProject::cleanup(){
	AppContextImpl::getApplicationContext()->setProject(NULL);
}


QList<XMLTestFactory*> ProjectTests::createTestFactories(){
	QList<XMLTestFactory*> res;
	res.append(GTest_LoadProject::createFactory());
	res.append(GTest_ExportProject::createFactory());
	res.append(GTest_UnloadProject::createFactory());
	res.append(GTest_LoadDocumentFromProject::createFactory());
	return res;
}

void GTest_ExportProject::init(XMLTestFormat* tf, const QDomElement& el) {
	Q_UNUSED(tf);
	
	exportTask = NULL;
	url = env->getVar("TEMP_DATA_DIR") + el.attribute("url");
}

void GTest_ExportProject::prepare(){
	QDir qd(url);
	if(qd.exists(url)){
		if(!removeDir(qd)){
			stateInfo.setError(tr("GTest_ExportProject::prepare unable to clean directory for exporting"));
			return;
		}
	}	
	if(!qd.mkpath(url)){
		stateInfo.setError(tr("GTest_ExportProject::prepare unable to create directory for exporting"));
		return;
	}
	exportTask = new ExportProjectTask(url);
	addSubTask(exportTask);
}

Task::ReportResult GTest_ExportProject::report(){
	if(exportTask!=NULL){
		if (exportTask->hasErrors()) {
			stateInfo.setError(exportTask->getError());
		}
	}
	return ReportResult_Finished;
}

void GTest_ExportProject::cleanup(){
	QDir qd(url);
	if(qd.exists(url)){
		if(!removeDir(QDir(url))){
			log.info(tr("GTest_ExportProject::cleanup unable to delete exported files"));
		}
	}
}

bool GTest_ExportProject::removeDir( const QDir &aDir ){
	bool has_err = false;
	if (aDir.exists())//QDir::NoDotAndDotDot
	{
		QFileInfoList entries = aDir.entryInfoList(QDir::NoDotAndDotDot | 
			QDir::Dirs | QDir::Files);
		int count = entries.size();
		for (int idx = 0; idx < count; idx++)
		{
			QFileInfo entryInfo = entries[idx];
			QString path = entryInfo.canonicalFilePath();
			if (entryInfo.isDir()){
				QDir aaa(path);
				has_err = removeDir( aaa );
			}else{
				QFile file(path);
				if (!file.remove())
					has_err = true;
			}
		}
		if (!aDir.rmdir(aDir.absolutePath()))
			has_err = true;
	}
	return(has_err);
}
void GTest_UnloadProject::init(XMLTestFormat* tf, const QDomElement& el) {
	Q_UNUSED(tf);

	QString packedList = el.attribute("documents");
	if(packedList.isEmpty()){
		//document list can be empty!
		return;
	}
	unloadDocList = packedList.split(",");
}

void GTest_UnloadProject::prepare(){
	Project *p = AppContext::getProject();
	if(p != NULL){
		foreach(QString doc, unloadDocList){
			removeContext(doc);
		}
		AppContextImpl::getApplicationContext()->setProject(NULL);
	}
}

void GTest_LoadDocumentFromProject::init(XMLTestFormat* tf, const QDomElement& el) {
    Q_UNUSED(tf);

	documentFileName = el.attribute("document");
	contextAdded = false;
}

void GTest_LoadDocumentFromProject::prepare(){
	Project *pr = AppContext::getProject();
	if(pr == NULL){
		stateInfo.setError(tr("No project loaded"));
		return;
	}
	if(documentFileName.isEmpty()){
		stateInfo.setError(tr("Document name to load is empty"));
		return;
	}
	foreach(Document *doc, pr->getDocuments()){
		QFileInfo fi(doc->getURL());
		QString tmp = fi.fileName();
		if (fi.fileName() == documentFileName){
			if(!doc->isLoaded()){
				loadTask = new LoadUnloadedDocumentTask(doc);
				addSubTask(loadTask);
			}
			loadedDoc = doc;
		}
	}
	if (loadedDoc == NULL){
		stateInfo.setError(tr("File %1 not found in project").arg(documentFileName));
		return;
	}
}

void GTest_LoadDocumentFromProject::cleanup(){
	QObject *o = getContext(documentFileName);
	if(contextAdded && o != NULL){
		removeContext(documentFileName);
		if(loadedDoc->isLoaded()){
			AppContext::getTaskScheduler()->registerTopLevelTask(new UnloadDocumentTask(loadedDoc, false));
		}
	}
}

Task::ReportResult GTest_LoadDocumentFromProject::report(){
	if(loadTask!=NULL){
		if(!loadTask->hasErrors()){
			addContext(documentFileName, loadedDoc);
			contextAdded = true;
		}
	}
	return ReportResult_Finished;
}

} //namespace
