/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _GB2_PROJECT_TASKS_H_
#define _GB2_PROJECT_TASKS_H_

#include <QUrl>

#include <core_api/Task.h>
#include <core_api/ResourceTracker.h>
#include <core_api/ProjectModel.h>
#include <test_framework/xmltest/XMLTestUtils.h>
#include <util_algorithm/GAutoDeleteList.h>
#include <core_api/AppResources.h>
#include <util_tasks/MultiTask.h>
#include <util_tasks/LoadDocumentTask.h>

class QDomDocument;


namespace GB2 {

typedef QString ProjectVersionId;

class Project;
class StateLock;

/// CloseProjectTask schema:
//	->CloseProjectTask (serial subtasks):
//		->SaveProjectTask
//		->UnregisterProjectTask	
class CloseProjectTask : public Task {
    Q_OBJECT

public:
	CloseProjectTask();
	virtual void prepare();
};


/// OpenProjectTask schema:
//	->OpenProjectTask (serial subtasks):
//		->CloseProjectTask
//		->LoadProjectTask
//		->RegisterProjectTask (added after LoadProjectTask is successfully finished)

class LoadProjectTask;
class OpenProjectTask : public Task {
    Q_OBJECT

public:
	OpenProjectTask(const QString& url, bool closeActiveProject, const QString& name = QString::null);
    OpenProjectTask(const QList<QUrl>& list, bool closeActiveProject);
	
	virtual void prepare();
	
protected:
	virtual QList<Task*> onSubTaskFinished(Task* subTask);

private:
	QString url;
	QString name;
    QList<QUrl> urlList;
	LoadProjectTask* loadProjectTask;
    bool closeActiveProject;
};

class OpenProjectTaskAndExecDialog : public OpenProjectTask {
    Q_OBJECT

public:
    OpenProjectTaskAndExecDialog(QDialog *_dialog);
    Task::ReportResult report();
private:
    QDialog *dialog;
};


/// SaveProjectTask schema:
//	->SaveProjectTask
//		->SaveDocumentsTask
class SaveProjectTask : public Task {
    Q_OBJECT

public:
    SaveProjectTask(SaveProjectTaskKind k, Project* p = NULL, const QString& url = QString::null);
    ~SaveProjectTask();

    virtual void prepare();
    ReportResult report();

private:
    SaveProjectTaskKind k;
    Project* proj;
    QString url;
};

typedef QMap<QString,QString> StrMap;

class SaveOnlyProjectTask : public Task {
    Q_OBJECT
public:
    SaveOnlyProjectTask(Project* p = NULL, const QString& url = QString::null);
    ~SaveOnlyProjectTask();

    virtual void prepare();
    ReportResult report();
    
    static void saveProjectFile(TaskStateInfo& ts, Project* p, const QString& projectUrl,   const StrMap& docUrlRemap = StrMap());

private:
    QList<Task*> onSubTaskFinished(Task* subTask);
    void _run();

    Task *sub;
    Project* proj;
    QString url;
    StateLock* lock;
    QList<Document*> phantomDocs;
};


/// LoadProjectTask
class LoadProjectTask : public Task {
    Q_OBJECT

public:
	LoadProjectTask(const QString& url);
	~LoadProjectTask();

	virtual void run();
	ReportResult report();

	Project* getProject() const {return proj;}
	Project* detachProject() {Project* tmp = proj; proj = NULL; return tmp;}

	static void loadXMLProjectModel(const QString& url, TaskStateInfo& si, QDomDocument& doc, ProjectVersionId &ver);

private:
	ProjectVersionId version;
	Project* proj;
	QString url;
	QDomDocument* xmlDoc;
};


/// register project task
//TODO: remove service+plugin binding -> add 'singleton' property to service. After its done this task will not be needed

class RegisterProjectServiceTask : public Task {
    Q_OBJECT
public:
    RegisterProjectServiceTask(Project* proj);
    void prepare();
private:
    Project* proj;
};

class ProjectParser : public QObject {
	Q_OBJECT
public:
	virtual Project* createProjectFromXMLModel(const QString& url, const QDomDocument& xmlDoc, TaskStateInfo& si) = 0;
	virtual ProjectVersionId getVersion(){assert(false); return QString();};
};

class ProjectParser10 : public ProjectParser {
	Q_OBJECT
public:
	ProjectParser10(){};
	Project* createProjectFromXMLModel(const QString& url, const QDomDocument& xmlDoc, TaskStateInfo& si);
	ProjectVersionId getVersion(){return QString("1.0");};
};

class ProjectParserRegistry {
public:
	ProjectParser* getProjectParserByVersion(ProjectVersionId id);
 	static ProjectParserRegistry* instance();

private:
    ProjectParserRegistry(ProjectParserRegistry &ppr){Q_UNUSED(ppr); assert(0);};
	ProjectParserRegistry(){init();};
	void init();
	GAutoDeleteList<ProjectParser> parsers;
};

class ExportProjectTask : public Task{
	Q_OBJECT
public:
	ExportProjectTask(QString _destinationDir, bool _compress = false);

	void prepare();
	ReportResult report();
private:
	void _run(TaskStateInfo& ts, Project* p, QString url, QStringList paths);

	bool compress;
	Project *project;
	QString destinationDir;
};


class GTest_LoadProject : public GTest {
	Q_OBJECT
public:
	SIMPLE_XML_TEST_BODY_WITH_FACTORY_EXT(GTest_LoadProject, "load-project", TaskFlags_NR_FOSCOE);

	void prepare();

	ReportResult report();

	virtual void cleanup();

	QList<Task*> onSubTaskFinished(Task* subTask);
private:
	QStringList			docContextList;
 	QString             projContextName;
	LoadProjectTask*    loadTask;
	MultiTask			*mt;
 	bool                contextAdded;
 	QString             url;	
};

class GTest_ExportProject : public GTest {
	Q_OBJECT
public:
	SIMPLE_XML_TEST_BODY_WITH_FACTORY_EXT(GTest_ExportProject, "export-project", TaskFlags_NR_FOSCOE);

	void prepare();

	ReportResult report();

	virtual void cleanup();
private:
	bool removeDir( const QDir &dir);
	QString             url;	
	QString				projContextname;
	ExportProjectTask   *exportTask;
};

class GTest_UnloadProject : public GTest {
	Q_OBJECT
public:
	SIMPLE_XML_TEST_BODY_WITH_FACTORY_EXT(GTest_UnloadProject, "unload-project", TaskFlags_NR_FOSCOE);

	void prepare();
private:
	QStringList unloadDocList;
};

class GTest_LoadDocumentFromProject : public GTest {
	Q_OBJECT
public:
	SIMPLE_XML_TEST_BODY_WITH_FACTORY_EXT(GTest_LoadDocumentFromProject, "load-document-from-project", TaskFlags_NR_FOSCOE);

	void prepare();
	void cleanup();
	ReportResult report();
private:
	LoadUnloadedDocumentTask *loadTask;
	Document* loadedDoc;
	QString documentFileName;
	bool contextAdded;
};

class ProjectTests {
public:
	static QList<XMLTestFactory*> createTestFactories();
};

}//namespace

#endif
